# cartmodels.py -- Qt Model for manipulation of openbabel OBMol data
#
# Copyright (c) Adam Tenderholt, Stanford University, 2005-2007
#                               a-tenderholt@stanford.edu
#
# This program is free software; you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version with the exceptions noted in the 
# Help dialog.

import sys
from math import sin, cos, sqrt

from PyQt4 import QtCore, QtGui
from cclib.parser import utils,ccopen
from cclib.bridge import makeopenbabel
import numpy
import openbabel
import pybel

from glwidget import *

table = utils.PeriodicTable()

class OBMolModel(QtCore.QAbstractItemModel):
    def __init__(self, parent = None):
        QtCore.QAbstractItemModel.__init__(self,parent)

        self.obmol = openbabel.OBMol()

    def headerData(self, section, orientation, role):
        if orientation == QtCore.Qt.Horizontal and role == QtCore.Qt.DisplayRole:
            if section == 0:
                return QtCore.QVariant("Element")
            if section == 1:
                return QtCore.QVariant("x")
            if section == 2:
                return QtCore.QVariant("y")
            if section == 3:
                return QtCore.QVariant("z")

        if orientation == QtCore.Qt.Vertical and role == QtCore.Qt.DisplayRole:
            return QtCore.QVariant(section + 1)

        return QtCore.QVariant()

    def flags(self, index):
        if not index.isValid():
            return QtCore.Qt.ItemIsEnabled

        return QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEditable | QtCore.Qt.ItemIsEnabled

    def rowCount(self, parent = QtCore.QModelIndex()):
        return self.obmol.NumAtoms()

    def columnCount(self, parent = QtCore.QModelIndex()):
        return 4

    def parent(self, index):
        return QtCore.QModelIndex()

    def index(self, row, column, parent = QtCore.QModelIndex()):
        return self.createIndex(row, column, id(self.obmol.GetAtom(row+1))+column)

    def data(self, index, role = QtCore.Qt.DisplayRole):
        if role != QtCore.Qt.DisplayRole:
            return QtCore.QVariant()
        atom = pybel.Atom(self.obmol.GetAtom(index.row()+1))
        vector = atom.coords
        num = atom.atomicnum

        if index.column() == 0:
            name = table.element[num]
            return QtCore.QVariant(name)

        if index.column() == 1:
            value = "%.3f"%(vector[0])
            return QtCore.QVariant(value)

        if index.column() == 2:
            value = "%.3f"%(vector[1])
            return QtCore.QVariant(value)

        if index.column() == 3:
            value = "%.3f"%(vector[2])
            return QtCore.QVariant(value)

        return QtCore.QVariant("error")

    def setData(self, index, value, role):
        atom = self.obmol.GetAtom(index.row()+1)
        vector = atom.GetVector()

        if index.column() == 0:
            name = str(value.toString())
            num = table.number[name]
            atom.SetAtomicNum(num)

        if index.column() == 1:
            pos = value.toDouble()[0]
            vector.SetX(pos)
            atom.SetVector(vector)

        if index.column() == 2:
            pos = value.toDouble()[0]
            vector.SetY(pos)
            atom.SetVector(vector)

        if index.column() == 3:
            pos = value.toDouble()[0]
            vector.SetZ(pos)
            atom.SetVector(vector)

        size = self.obmol.NumAtoms()
        topLeft = self.index(0,0)
        bottomRight = self.index(size,3)

        self.emit(QtCore.SIGNAL("dataChanged(const QModelIndex&, const QModelIndex&)"),topLeft,bottomRight)

        return True
        print "Did we really make it this far?"

    def centerMolecule(self, index):
        atom = pybel.Atom(self.obmol.GetAtom(index+1))
        x = atom.coords[0]
        y = atom.coords[1]
        z = atom.coords[2]

        size = self.obmol.NumAtoms()

        for i in range(size):
            atom = pybel.Atom(self.obmol.GetAtom(i+1))
            atom.OBAtom.SetVector(atom.coords[0] - x, atom.coords[1] - y, atom.coords[2] - z)

        topLeft = self.index(0,0)
        bottomRight = self.index(size,3)

        self.emit(QtCore.SIGNAL("dataChanged(const QModelIndex&, const QModelIndex&)"),topLeft,bottomRight)

    def rotateMolecule(self, angle, rotvec, transvec = [0, 0, 0]):

        rotmat = numpy.zeros((3,3),"f")

        x = rotvec[0]
        y = rotvec[1]
        z = rotvec[2]

        length = sqrt(x**2 + y**2 + z**2)
        x /= length
        y /= length
        z /= length

        costh = cos(angle)
        sinth = sin(angle)

        rotmat[0,0] = costh + (1 - costh) * x**2
        rotmat[0,1] = (1 - costh) * x * y - sinth * z
        rotmat[0,2] = (1 - costh) * x * z + sinth*y
        rotmat[1,0] = (1 - costh) * x * y + sinth * z
        rotmat[1,1] = costh + (1 - costh) * y**2
        rotmat[1,2] = (1 - costh) * y * z - sinth * x
        rotmat[2,0] = (1 - costh) * z * x - sinth * y
        rotmat[2,1] = (1 - costh) * z * y + sinth * x
        rotmat[2,2] = costh + (1 - costh) * z**2

        size = self.obmol.NumAtoms()
        for j in range(size):

            atom = self.obmol.GetAtom(j+1)
            #rotate positions
            vector = pybel.Atom(atom).coords
            oldPos = numpy.array([vector[0], vector[1], vector[2]])
            newPos = numpy.inner(rotmat, oldPos)

            atom.SetVector(newPos[0], newPos[1], newPos[2])

        topLeft = self.index(0,0)
        bottomRight = self.index(size,3)

        self.emit(QtCore.SIGNAL("dataChanged(const QModelIndex&, const QModelIndex&)"),topLeft,bottomRight)

    def slotTimer(self):
        self.rotateMolecule(10, [1, 0, 1])

    def setMolecule(self, molecule):
        self.obmol = molecule

        topLeft = self.index(0,0)
        size = self.obmol.NumAtoms()
        bottomRight = self.index(size,3)

        self.emit(QtCore.SIGNAL("dataChanged(const QModelIndex&, const QModelIndex&)"),topLeft,bottomRight)


    def addAtom(self):
        size = self.obmol.NumAtoms()

        self.beginInsertRows(QtCore.QModelIndex(), size, size+1)
        newatom = self.obmol.NewAtom()
        newatom.SetAtomicNum(6)
        newatom.SetVector(0.0, 0.0, 0.0)
        self.endInsertRows()

        topLeft = self.index(size-1,0)
        bottomRight = self.index(size, 3)

        self.emit(QtCore.SIGNAL("dataChanged(const QModelIndex&, const QModelIndex&)"),topLeft, bottomRight)

    def removeAtom(self, i):

        self.beginRemoveRows(QtCore.QModelIndex(), i, i)
        atom = self.obmol.GetAtom(i+1)
        self.obmol.DeleteAtom(atom)
        self.endRemoveRows()

        size = self.obmol.NumAtoms()
        topLeft = self.index(i,0)
        bottomRight = self.index(size,3)
        self.emit(QtCore.SIGNAL("dataChanged(const QModelIndex&, const QModelIndex&)"),topLeft, bottomRight)

if __name__ == '__main__':
    
    app = QtGui.QApplication(sys.argv)

    parser=ccopen(sys.argv[1])
    parser.parse()
    molecule = makeopenbabel(parser.atomcoords[0],parser.atomnos)

    view = QtGui.QTableView()
    model = OBMolModel()

    view.setModel(model)
    view.setWindowTitle("Atoms")
    model.setMolecule(molecule)
    view.show()

    timer = QtCore.QTimer()
    QtCore.QObject.connect(timer, QtCore.SIGNAL("timeout()"),model.slotTimer)
    timer.start(1000)

    glwidget = GLWidget()
    glwidget.obmol = molecule
    glwidget.show()

    QtCore.QObject.connect(model,QtCore.SIGNAL("dataChanged(const QModelIndex&, const QModelIndex&)"),glwidget.slotDataChanged)

    sys.exit(app.exec_())
