# popwidget.py -- widget used in the population methods tab
#
# Copyright (c) Adam Tenderholt, Stanford University, 2005-2007
#                               a-tenderholt@stanford.edu
#
# This program is free software; you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version with the exceptions noted in the 
# Help dialog.

import sys
from os.path import exists
import platform as plat

from PyQt4 import QtCore, QtGui
from cclib.parser import ccopen, utils

from popmodels import *
from glwidget import *

table = utils.PeriodicTable()

class PopWidget(QtGui.QWidget):
    def __init__(self, parent = None, flags = 0):
        QtGui.QWidget.__init__(self)

        self.setParent(parent)

#setup widgets
        self.gridlayout = QtGui.QGridLayout(self)
        self.gridlayout.setMargin(9)
        self.gridlayout.setSpacing(6)
        self.gridlayout.setObjectName("gridlayout2")

        self.splitter = QtGui.QSplitter(self)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")

        hboxlayout = QtGui.QHBoxLayout(self.splitter)

#setup Tree views
        view = QtGui.QTreeView(self.splitter)
        view.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
        view.setDragEnabled(True)
        view.show()

        fragview = FragView(self.splitter) #QtGui.QTreeView()
        fragview.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
        fragview.setDragEnabled(True)
        fragview.setAcceptDrops(True)
        fragview.setDropIndicatorShown(True)
        fragview.show()

#setup models
        self.aomodel = AOModel(self)
        view.setModel(self.aomodel)
        self.selectionModel = QtGui.QItemSelectionModel(self.aomodel)
        view.setSelectionModel(self.selectionModel)
        self.fragmodel = FragModel(self)
        fragview.setModel(self.fragmodel)

        hboxlayout.addWidget(view)
        hboxlayout.addWidget(fragview)

        glWidget = GLWidget(self.splitter)
        glWidget.resize(600,400)

        self.splitter.setSizes([75,75,250])
        self.gridlayout.addWidget(self.splitter)

#setup slots
        QtCore.QObject.connect(self.selectionModel,QtCore.SIGNAL("selectionChanged(const QItemSelection&, const QItemSelection&)"), self.changeSelection)

        if plat.system() != "Darwin":
            QtCore.QObject.connect(self.fragmodel, QtCore.SIGNAL("dataChanged()"), self.aomodel.slotDeleteMoved)
            QtCore.QObject.connect(self.fragmodel, QtCore.SIGNAL("dataChanged()"), self.fragmodel.slotDeleteMoved)

#setup class variables
        self.view = view
        self.fragview = fragview
        self.glWidget = glWidget
        self.data = None
        self.names = None

    def setData(self, data):
        
        if not ( hasattr(data,"fonames") or hasattr(data,"aonames") ):
            return

        if hasattr(data, "fonames"):
            self.names = data.fonames
            self.aomodel.addOrbitals(data.fonames)

        if hasattr(data, "aonames"):
            self.names = data.aonames
            self.aomodel.addOrbitals(data.aonames)

        if hasattr(data, "atomnos") and hasattr(data,"atomcoords"):
            self.glWidget.setMolecule(data.atomcoords[0],data.atomnos)

        self.view.setModel(self.aomodel)

        self.data = data

    def changeSelection(self, current, old):
        selection = {}
        isADF = ( str(self.data).split()[0] == "ADF" )

        for index in self.selectionModel.selectedIndexes():
            parentIndex = index.parent()

            if parentIndex == QtCore.QModelIndex():
                text = self.aomodel.data(index, QtCore.Qt.DisplayRole).toString()
            else:
                text = self.aomodel.data(parentIndex, QtCore.Qt.DisplayRole).toString()
            
            lindex = text.indexOf(")")
            infos = text[lindex+2:].split("+", QtCore.QString.SkipEmptyParts)

            regexp = QtCore.QRegExp("[1-9]")

            for info in infos:
                if not isADF:
                    index = regexp.indexIn(info)
                    value = int(info[index:])
                elif self.isElement(str(info[:2])):
                    value = int(info[2:])
                elif self.isElement(str(info[0])):
                    value = int(info[1:])
                else:
                    for fragname in self.data.fragnames:
                        name = fragname.split("_")[0]
                        index = info.indexOf(name)
                        if index == 0:
                            value = int(info[len(name)])

                selection[value - 1] = 1

        if isADF:
            selectionList = []
            for key in selection.keys():
                selectionList += self.data.frags[key]
            
            self.glWidget.setSelected(selectionList)
        else:
            self.glWidget.setSelected(selection.keys())

    def isElement(self, text):
        try:
            table.number[text]
        except KeyError:
            return False
        else:
            return True

if __name__=='__main__':
    app = QtGui.QApplication(sys.argv)
    window = PopWidget()
    window.resize(800,600)
    window.show()
    if len(sys.argv)==2 and exists(sys.argv[1]):
       parser = ccopen(sys.argv[1])
       data = parser.parse()
       window.setData(data)
       window.fragmodel.addFragment()
       window.fragmodel.addFragment()

    sys.exit(app.exec_())

